@@ -17,24 +17,12 @@ import uuid
 
 import fixtures
 from keystoneauth1 import fixture
-from keystoneclient.common import cms
-from keystoneclient import utils
 from oslo_serialization import jsonutils
-import six
 import testresources
 
 
 TESTDIR = os.path.dirname(os.path.abspath(__file__))
 ROOTDIR = os.path.normpath(os.path.join(TESTDIR, '..', '..', '..'))
-CERTDIR = os.path.join(ROOTDIR, 'examples', 'pki', 'certs')
-CMSDIR = os.path.join(ROOTDIR, 'examples', 'pki', 'cms')
-KEYDIR = os.path.join(ROOTDIR, 'examples', 'pki', 'private')
-
-
-def _hash_signed_token_safe(signed_text, **kwargs):
-    if isinstance(signed_text, six.text_type):
-        signed_text = signed_text.encode('utf-8')
-    return utils.hash_signed_token(signed_text, **kwargs)
 
 
 class Examples(fixtures.Fixture):
@@ -55,60 +43,14 @@ class Examples(fixtures.Fixture):
     def setUp(self):
         super(Examples, self).setUp()
 
-        # The data for several tests are signed using openssl and are stored in
-        # files in the signing subdirectory.  In order to keep the values
-        # consistent between the tests and the signed documents, we read them
-        # in for use in the tests.
-        with open(os.path.join(CMSDIR, 'auth_token_scoped.json')) as f:
-            self.TOKEN_SCOPED_DATA = cms.cms_to_token(f.read())
-
-        with open(os.path.join(CMSDIR, 'auth_token_scoped.pem')) as f:
-            self.SIGNED_TOKEN_SCOPED = cms.cms_to_token(f.read())
-        self.SIGNED_TOKEN_SCOPED_HASH = _hash_signed_token_safe(
-            self.SIGNED_TOKEN_SCOPED)
-        self.SIGNED_TOKEN_SCOPED_HASH_SHA256 = _hash_signed_token_safe(
-            self.SIGNED_TOKEN_SCOPED, mode='sha256')
-        with open(os.path.join(CMSDIR, 'auth_token_unscoped.pem')) as f:
-            self.SIGNED_TOKEN_UNSCOPED = cms.cms_to_token(f.read())
-        with open(os.path.join(CMSDIR, 'auth_v3_token_scoped.pem')) as f:
-            self.SIGNED_v3_TOKEN_SCOPED = cms.cms_to_token(f.read())
-        self.SIGNED_v3_TOKEN_SCOPED_HASH = _hash_signed_token_safe(
-            self.SIGNED_v3_TOKEN_SCOPED)
-        self.SIGNED_v3_TOKEN_SCOPED_HASH_SHA256 = _hash_signed_token_safe(
-            self.SIGNED_v3_TOKEN_SCOPED, mode='sha256')
-        with open(os.path.join(CMSDIR, 'auth_token_scoped_expired.pem')) as f:
-            self.SIGNED_TOKEN_SCOPED_EXPIRED = cms.cms_to_token(f.read())
-        with open(os.path.join(CMSDIR, 'auth_token_scoped.pkiz')) as f:
-            self.SIGNED_TOKEN_SCOPED_PKIZ = cms.cms_to_token(f.read())
-        with open(os.path.join(CMSDIR, 'auth_token_unscoped.pkiz')) as f:
-            self.SIGNED_TOKEN_UNSCOPED_PKIZ = cms.cms_to_token(f.read())
-        with open(os.path.join(CMSDIR, 'auth_v3_token_scoped.pkiz')) as f:
-            self.SIGNED_v3_TOKEN_SCOPED_PKIZ = cms.cms_to_token(f.read())
-        with open(os.path.join(CMSDIR,
-                               'auth_token_scoped_expired.pkiz')) as f:
-            self.SIGNED_TOKEN_SCOPED_EXPIRED_PKIZ = cms.cms_to_token(f.read())
-
-        self.SIGNING_CERT_FILE = os.path.join(CERTDIR, 'signing_cert.pem')
-        with open(self.SIGNING_CERT_FILE) as f:
-            self.SIGNING_CERT = f.read()
-
         self.KERBEROS_BIND = 'USER@REALM'
         self.SERVICE_KERBEROS_BIND = 'SERVICE_USER@SERVICE_REALM'
 
-        self.SIGNING_KEY_FILE = os.path.join(KEYDIR, 'signing_key.pem')
-        with open(self.SIGNING_KEY_FILE) as f:
-            self.SIGNING_KEY = f.read()
-
-        self.SIGNING_CA_FILE = os.path.join(CERTDIR, 'cacert.pem')
-        with open(self.SIGNING_CA_FILE) as f:
-            self.SIGNING_CA = f.read()
-
         self.UUID_TOKEN_DEFAULT = "ec6c0710ec2f471498484c1b53ab4f9d"
         self.UUID_TOKEN_NO_SERVICE_CATALOG = '8286720fbe4941e69fa8241723bb02df'
         self.UUID_TOKEN_UNSCOPED = '731f903721c14827be7b2dc912af7776'
         self.UUID_TOKEN_BIND = '3fc54048ad64405c98225ce0897af7c5'
         self.UUID_TOKEN_UNKNOWN_BIND = '8885fdf4d42e4fb9879e6379fa1eaf48'
-        self.VALID_DIABLO_TOKEN = 'b0cf19b55dbb4f20a6ee18e6c6cf1726'
         self.v3_UUID_TOKEN_DEFAULT = '5603457654b346fdbb93437bfe76f2f1'
         self.v3_UUID_TOKEN_UNSCOPED = 'd34835fdaec447e695a0a024d84f8d79'
         self.v3_UUID_TOKEN_DOMAIN_SCOPED = 'e8a7b63aaa4449f38f0c5c05c3581792'
@@ -121,57 +63,11 @@ class Examples(fixtures.Fixture):
         self.v3_UUID_SERVICE_TOKEN_DEFAULT = 'g431071bbc2f492748596c1b53cb229'
         self.v3_UUID_SERVICE_TOKEN_BIND = 'be705e4426d0449a89e35ae21c380a05'
         self.v3_NOT_IS_ADMIN_PROJECT = uuid.uuid4().hex
-        self.SIGNED_TOKEN_SCOPED_KEY = cms.cms_hash_token(
-            self.SIGNED_TOKEN_SCOPED)
-        self.SIGNED_TOKEN_UNSCOPED_KEY = cms.cms_hash_token(
-            self.SIGNED_TOKEN_UNSCOPED)
-        self.SIGNED_v3_TOKEN_SCOPED_KEY = cms.cms_hash_token(
-            self.SIGNED_v3_TOKEN_SCOPED)
 
-        self.SIGNED_TOKEN_SCOPED_PKIZ_KEY = cms.cms_hash_token(
-            self.SIGNED_TOKEN_SCOPED_PKIZ)
-        self.SIGNED_TOKEN_UNSCOPED_PKIZ_KEY = cms.cms_hash_token(
-            self.SIGNED_TOKEN_UNSCOPED_PKIZ)
-        self.SIGNED_v3_TOKEN_SCOPED_PKIZ_KEY = cms.cms_hash_token(
-            self.SIGNED_v3_TOKEN_SCOPED_PKIZ)
-
-        self.INVALID_SIGNED_TOKEN = (
-            "MIIAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
-            "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB"
-            "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC"
-            "DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD"
-            "EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE"
-            "FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
-            "0000000000000000000000000000000000000000000000000000000000000000"
-            "1111111111111111111111111111111111111111111111111111111111111111"
-            "2222222222222222222222222222222222222222222222222222222222222222"
-            "3333333333333333333333333333333333333333333333333333333333333333"
-            "4444444444444444444444444444444444444444444444444444444444444444"
-            "5555555555555555555555555555555555555555555555555555555555555555"
-            "6666666666666666666666666666666666666666666666666666666666666666"
-            "7777777777777777777777777777777777777777777777777777777777777777"
-            "8888888888888888888888888888888888888888888888888888888888888888"
-            "9999999999999999999999999999999999999999999999999999999999999999"
-            "0000000000000000000000000000000000000000000000000000000000000000")
-
-        self.INVALID_SIGNED_PKIZ_TOKEN = (
-            "PKIZ_AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
-            "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB"
-            "CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC"
-            "DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD"
-            "EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE"
-            "FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
-            "0000000000000000000000000000000000000000000000000000000000000000"
-            "1111111111111111111111111111111111111111111111111111111111111111"
-            "2222222222222222222222222222222222222222222222222222222222222222"
-            "3333333333333333333333333333333333333333333333333333333333333333"
-            "4444444444444444444444444444444444444444444444444444444444444444"
-            "5555555555555555555555555555555555555555555555555555555555555555"
-            "6666666666666666666666666666666666666666666666666666666666666666"
-            "7777777777777777777777777777777777777777777777777777777777777777"
-            "8888888888888888888888888888888888888888888888888888888888888888"
-            "9999999999999999999999999999999999999999999999999999999999999999"
-            "0000000000000000000000000000000000000000000000000000000000000000")
+        self.v3_APP_CRED_TOKEN = '6f506fa9641448bbaecbd12dd30678a9'
+        self.v3_APP_CRED_ACCESS_RULES = 'c417747898c44629b08791f2579e40a5'
+        self.v3_APP_CRED_EMPTY_ACCESS_RULES = 'c75905c307f04fdd9979126582d7aae'
+        self.v3_APP_CRED_MATCHING_RULES = 'ad49decc7106489d95ca9ed874b6cb66'
 
         # JSON responses keyed by token ID
         self.TOKEN_RESPONSES = {}
@@ -195,30 +91,12 @@ class Examples(fixtures.Fixture):
         SERVICE_ROLE_NAME1 = 'service'
         SERVICE_ROLE_NAME2 = 'service_role2'
 
+        APP_CRED_ID = 'app_cred_id1'
+
         self.SERVICE_TYPE = 'identity'
         self.UNVERSIONED_SERVICE_URL = 'https://keystone.example.com:1234/'
         self.SERVICE_URL = self.UNVERSIONED_SERVICE_URL + 'v2.0'
 
-        # Old Tokens
-
-        self.TOKEN_RESPONSES[self.VALID_DIABLO_TOKEN] = {
-            'access': {
-                'token': {
-                    'id': self.VALID_DIABLO_TOKEN,
-                    'expires': '2020-01-01T00:00:10.000123Z',
-                    'tenantId': PROJECT_ID,
-                },
-                'user': {
-                    'id': USER_ID,
-                    'name': USER_NAME,
-                    'roles': [
-                        {'name': ROLE_NAME1},
-                        {'name': ROLE_NAME2},
-                    ],
-                },
-            },
-        }
-
         # Generated V2 Tokens
 
         token = fixture.V2Token(token_id=self.UUID_TOKEN_DEFAULT,
@@ -246,20 +124,6 @@ class Examples(fixtures.Fixture):
         token.add_role(ROLE_NAME2)
         self.TOKEN_RESPONSES[self.UUID_TOKEN_NO_SERVICE_CATALOG] = token
 
-        token = fixture.V2Token(token_id=self.SIGNED_TOKEN_SCOPED_KEY,
-                                tenant_id=PROJECT_ID,
-                                tenant_name=PROJECT_NAME,
-                                user_id=USER_ID,
-                                user_name=USER_NAME)
-        token.add_role(ROLE_NAME1)
-        token.add_role(ROLE_NAME2)
-        self.TOKEN_RESPONSES[self.SIGNED_TOKEN_SCOPED_KEY] = token
-
-        token = fixture.V2Token(token_id=self.SIGNED_TOKEN_UNSCOPED_KEY,
-                                user_id=USER_ID,
-                                user_name=USER_NAME)
-        self.TOKEN_RESPONSES[self.SIGNED_TOKEN_UNSCOPED_KEY] = token
-
         token = fixture.V2Token(token_id=self.UUID_TOKEN_BIND,
                                 tenant_id=PROJECT_ID,
                                 tenant_name=PROJECT_NAME,
@@ -359,7 +223,6 @@ class Examples(fixtures.Fixture):
         token.add_role(name=ROLE_NAME2)
         svc = token.add_service(self.SERVICE_TYPE)
         svc.add_endpoint('public', self.SERVICE_URL)
-        self.TOKEN_RESPONSES[self.SIGNED_v3_TOKEN_SCOPED_KEY] = token
 
         token = fixture.V3Token(user_id=USER_ID,
                                 user_name=USER_NAME,
@@ -437,14 +300,122 @@ class Examples(fixtures.Fixture):
         svc.add_endpoint('public', self.SERVICE_URL)
         self.TOKEN_RESPONSES[self.v3_NOT_IS_ADMIN_PROJECT] = token
 
-        # PKIZ tokens generally link to above tokens
+        # Application credential token
+        token = fixture.V3Token(user_id=USER_ID,
+                                user_name=USER_NAME,
+                                user_domain_id=DOMAIN_ID,
+                                user_domain_name=DOMAIN_NAME,
+                                project_id=PROJECT_ID,
+                                project_name=PROJECT_NAME,
+                                project_domain_id=DOMAIN_ID,
+                                project_domain_name=DOMAIN_NAME,
+                                application_credential_id=APP_CRED_ID)
+        token.add_role(name=ROLE_NAME1)
+        token.add_role(name=ROLE_NAME2)
+        svc = token.add_service(self.SERVICE_TYPE)
+        svc.add_endpoint('public', self.SERVICE_URL)
+        svc = token.add_service('compute')
+        svc.add_endpoint('public', 'https://nova.openstack.example.org/v2.1')
+        self.TOKEN_RESPONSES[self.v3_APP_CRED_TOKEN] = token
 
-        self.TOKEN_RESPONSES[self.SIGNED_TOKEN_SCOPED_PKIZ_KEY] = (
-            self.TOKEN_RESPONSES[self.SIGNED_TOKEN_SCOPED_KEY])
-        self.TOKEN_RESPONSES[self.SIGNED_TOKEN_UNSCOPED_PKIZ_KEY] = (
-            self.TOKEN_RESPONSES[self.SIGNED_TOKEN_UNSCOPED_KEY])
-        self.TOKEN_RESPONSES[self.SIGNED_v3_TOKEN_SCOPED_PKIZ_KEY] = (
-            self.TOKEN_RESPONSES[self.SIGNED_v3_TOKEN_SCOPED_KEY])
+        # Application credential with access_rules token
+        access_rules = [{
+            'path': '/v2.1/servers',
+            'method': 'GET',
+            'service': 'compute'
+        }]
+        token = fixture.V3Token(
+            user_id=USER_ID,
+            user_name=USER_NAME,
+            user_domain_id=DOMAIN_ID,
+            user_domain_name=DOMAIN_NAME,
+            project_id=PROJECT_ID,
+            project_name=PROJECT_NAME,
+            project_domain_id=DOMAIN_ID,
+            project_domain_name=DOMAIN_NAME,
+            application_credential_id=APP_CRED_ID,
+            application_credential_access_rules=access_rules)
+        token.add_role(name=ROLE_NAME1)
+        token.add_role(name=ROLE_NAME2)
+        svc = token.add_service(self.SERVICE_TYPE)
+        svc.add_endpoint('public', self.SERVICE_URL)
+        svc = token.add_service('compute')
+        svc.add_endpoint('public', 'https://nova.openstack.example.org')
+        svc = token.add_service('image')
+        svc.add_endpoint('public', 'https://glance.openstack.example.org')
+        self.TOKEN_RESPONSES[self.v3_APP_CRED_ACCESS_RULES] = token
+
+        # Application credential with explicitly empty access_rules
+        access_rules = []
+        token = fixture.V3Token(
+            user_id=USER_ID,
+            user_name=USER_NAME,
+            user_domain_id=DOMAIN_ID,
+            user_domain_name=DOMAIN_NAME,
+            project_id=PROJECT_ID,
+            project_name=PROJECT_NAME,
+            project_domain_id=DOMAIN_ID,
+            project_domain_name=DOMAIN_NAME,
+            application_credential_id=APP_CRED_ID,
+            application_credential_access_rules=access_rules)
+        token.add_role(name=ROLE_NAME1)
+        token.add_role(name=ROLE_NAME2)
+        svc = token.add_service(self.SERVICE_TYPE)
+        svc.add_endpoint('public', self.SERVICE_URL)
+        self.TOKEN_RESPONSES[self.v3_APP_CRED_EMPTY_ACCESS_RULES] = token
+
+        # Application credential with matching rules
+        access_rules = [
+            {
+                'path': '/v2.1/servers/{server_id}',
+                'method': 'GET',
+                'service': 'compute'
+            },
+            {
+                'path': '/v2/images/*',
+                'method': 'GET',
+                'service': 'image'
+            },
+            {
+                'path': '**',
+                'method': 'GET',
+                'service': 'identity'
+            },
+            {
+                'path': '/v3/{project_id}/types/{volume_type_id}',
+                'method': 'GET',
+                'service': 'block-storage'
+            },
+            {
+                'path': '/v1/*/*/*',
+                'method': 'GET',
+                'service': 'object-store'
+            }
+        ]
+        token = fixture.V3Token(
+            user_id=USER_ID,
+            user_name=USER_NAME,
+            user_domain_id=DOMAIN_ID,
+            user_domain_name=DOMAIN_NAME,
+            project_id=PROJECT_ID,
+            project_name=PROJECT_NAME,
+            project_domain_id=DOMAIN_ID,
+            project_domain_name=DOMAIN_NAME,
+            application_credential_id=APP_CRED_ID,
+            application_credential_access_rules=access_rules)
+        token.add_role(name=ROLE_NAME1)
+        token.add_role(name=ROLE_NAME2)
+        svc = token.add_service(self.SERVICE_TYPE)
+        svc.add_endpoint('public', self.SERVICE_URL)
+        svc = token.add_service('compute')
+        svc.add_endpoint('public', 'https://nova.openstack.example.org')
+        svc = token.add_service('image')
+        svc.add_endpoint('public', 'https://glance.openstack.example.org')
+        svc = token.add_service('block-storage')
+        svc.add_endpoint('public', 'https://cinder.openstack.example.org')
+        svc = token.add_service('object-store')
+        svc.add_endpoint('public', 'https://swift.openstack.example.org')
+        self.TOKEN_RESPONSES[self.v3_APP_CRED_MATCHING_RULES] = token
 
         self.JSON_TOKEN_RESPONSES = dict([(k, jsonutils.dumps(v)) for k, v in
                                           self.TOKEN_RESPONSES.items()])
